"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vitest = __importStar(require("vitest"));
const rule_tester_1 = require("@typescript-eslint/rule-tester");
const enforce_consistent_nullability_1 = __importDefault(require("./enforce-consistent-nullability"));
rule_tester_1.RuleTester.afterAll = vitest.afterAll;
rule_tester_1.RuleTester.it = vitest.it;
rule_tester_1.RuleTester.itOnly = vitest.it.only;
rule_tester_1.RuleTester.describe = vitest.describe;
const ruleTester = new rule_tester_1.RuleTester({
    parser: '@typescript-eslint/parser',
});
ruleTester.run('enforce-consistent-nullability', enforce_consistent_nullability_1.default, {
    valid: [
        {
            name: 'should check column nullable if the nullability is always',
            code: `class Entity {
                @Column({ type: 'string', nullable: false })
                str: string;

                @Column({ type: 'string', nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'always' }],
        },
        {
            name: 'should check column nullable if the nullability is non-default',
            code: `class Entity {
                @Column({ type: 'string' })
                str: string;

                @Column({ type: 'string', nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
        },
        {
            name: 'should check relation nullable if the nullability is always',
            code: `class Entity {
                @ManyToOne(() => Other, { nullable: false })
                other: Other;

                @ManyToOne(() => Other, { nullable: true })
                other: Other | null;
            }`,
            options: [{ specifyNullable: 'always' }],
        },
        {
            name: 'should check relation nullable if the nullability is non-default',
            code: `class Entity {
                @ManyToOne(() => Other, { nullable: false })
                other: Other;

                @ManyToOne(() => Other)
                other: Other | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
        },
    ],
    invalid: [
        {
            name: 'should fail undefined column nullable if the nullability is always',
            code: `class Entity {
                @Column({ type: 'string' })
                str: string;

                @Column({ type: 'string', nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'always' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_missing_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_set_nullable',
                            output: `class Entity {
                @Column({ type: 'string', nullable: false })
                str: string;

                @Column({ type: 'string', nullable: true })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail undefined string nullable if the nullability is always',
            code: `class Entity {
                @Column('string', {})
                str: string;

                @Column('string', { nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'always' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_missing_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_set_nullable',
                            output: `class Entity {
                @Column('string', { nullable: false })
                str: string;

                @Column('string', { nullable: true })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail undefined unset nullable if the nullability is always',
            code: `class Entity {
                @Column()
                str: string;

                @Column({ nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'always' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_missing_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_set_nullable',
                            output: `class Entity {
                @Column({ nullable: false })
                str: string;

                @Column({ nullable: true })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail given column nullable if the nullability is non-default',
            code: `class Entity {
                @Column({ type: 'string', nullable: false })
                str: string;

                @Column({ type: 'string', nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_superfluous_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_remove_nullable',
                            output: `class Entity {
                @Column({ type: 'string' })
                str: string;

                @Column({ type: 'string', nullable: true })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail given unordered column nullable if the nullability is non-default',
            code: `class Entity {
                @Column({ nullable: false, type: 'string' })
                str: string;

                @Column({ nullable: true, type: 'string' })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_superfluous_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_remove_nullable',
                            output: `class Entity {
                @Column({ type: 'string' })
                str: string;

                @Column({ nullable: true, type: 'string' })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail given string column nullable if the nullability is non-default',
            code: `class Entity {
                @Column('string', { nullable: false })
                str: string;

                @Column('string', { nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_superfluous_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_remove_nullable',
                            output: `class Entity {
                @Column('string')
                str: string;

                @Column('string', { nullable: true })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail given unknown column nullable if the nullability is non-default',
            code: `class Entity {
                @Column({ nullable: false })
                str: string;

                @Column({ nullable: true })
                str: string | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_superfluous_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_remove_nullable',
                            output: `class Entity {
                @Column()
                str: string;

                @Column({ nullable: true })
                str: string | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail undefined relation nullable if the nullability is always',
            code: `class Entity {
                @ManyToOne(() => Other, { nullable: false })
                other: Other;

                @ManyToOne(() => Other)
                other: Other | null;
            }`,
            options: [{ specifyNullable: 'always' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_missing_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_set_nullable',
                            output: `class Entity {
                @ManyToOne(() => Other, { nullable: false })
                other: Other;

                @ManyToOne(() => Other, { nullable: true })
                other: Other | null;
            }`,
                        },
                    ],
                },
            ],
        },
        {
            name: 'should fail given relation nullable if the nullability is non-default',
            code: `class Entity {
                @ManyToOne(() => Other, { nullable: false })
                other: Other;

                @ManyToOne(() => Other, { nullable: true })
                other: Other | null;
            }`,
            options: [{ specifyNullable: 'non-default' }],
            errors: [
                {
                    messageId: 'typescript_typeorm_superfluous_nullability',
                    suggestions: [
                        {
                            messageId: 'typescript_typeorm_remove_nullable',
                            output: `class Entity {
                @ManyToOne(() => Other, { nullable: false })
                other: Other;

                @ManyToOne(() => Other)
                other: Other | null;
            }`,
                        },
                    ],
                },
            ],
        },
    ],
});
